@@ -28,6 +28,7 @@ from iommi.part import Part
 from iommi.refinable import (
     EvaluatedRefinable,
     Refinable,
+    SpecialEvaluatedRefinable,
 )
 from iommi.shortcut import with_defaults
 
@@ -42,29 +43,48 @@ class Action(Fragment):
 
     .. code-block:: python
 
+        # @test
+        actions = [
+        # @end
+
         # Link
-        Action(attrs__href='http://example.com')
+        Action(attrs__href='http://example.com'),
 
         # Link with icon
-        Action.icon('edit', attrs__href="edit/")
+        Action.icon('edit', attrs__href="edit/"),
 
         # Button
-        Action.button(display_name='Button title!')
+        Action.button(display_name='Button title!'),
 
         # A submit button
-        Action.submit(display_name='Do this')
+        Action.submit(display_name='Do this'),
 
         # The primary submit button on a form.
-        Action.primary()
+        Action.primary(),
 
-        # Notice that because forms
-        # with a single primary submit button are so common, iommi assumes
-        # that if you have a action called submit and do NOT explicitly
-        # specify the action that it is a primary action. This is only
-        # done for the action called submit, inside the Forms actions
-        # Namespace.
-        #
-        # For that reason this works:
+        # @test
+        ]
+
+        from iommi.refinable import Prio
+        foo = Page(
+            parts={
+                f'button_{i}': html.div(action.refine(display_name='Action', prio=Prio.shortcut))
+                for i, action in enumerate(actions)
+            }
+        )
+        show_output(foo)
+        # @end
+
+    Notice that because forms
+    with a single primary submit button are so common, iommi assumes
+    that if you have an action called submit and do NOT explicitly
+    specify the action that it is a primary action. This is only
+    done for the action called submit, inside the Forms actions
+    Namespace.
+
+    For that reason this works:
+
+    .. code-block:: python
 
         class MyForm(Form):
             class Meta:
@@ -75,7 +95,9 @@ class Action(Fragment):
 
                     ...
 
-        # and is roughly equivalent to
+    and is roughly equivalent to
+
+    .. code-block:: python
 
         def on_submit(form, **_):
             if not form.is_valid():
@@ -187,6 +209,5 @@ def group_actions(actions: Dict[str, Action]):
 
 
 class Actions(Members, Tag):
-    # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
-    attrs: Attrs = Refinable()
+    attrs: Attrs = SpecialEvaluatedRefinable()
     tag = EvaluatedRefinable()
