@@ -361,6 +361,20 @@ class Admin(Page):
                 .table_class.get_meta()
                 .member_class(
                     cell__value=lambda row, **_: f'apps__{row.key}__include = True' if row.key else '',
+                    cell__format=lambda value, **_: (
+                        format_html(
+                            '''
+                                {}
+                                <span style="margin-left: 1rem" onclick="navigator.clipboard.writeText('{}')">
+                                    <i class="fa fa-copy"></i>
+                                </span>
+                            ''',
+                            value,
+                            value,
+                        )
+                        if value
+                        else ''
+                    ),
                     # cell__url=lambda row, **_: f'{row.url}add_model/' if row.key else None,
                     after=LAST,
                 )
@@ -382,8 +396,8 @@ class Admin(Page):
 
                             Read <a href="https://docs.iommi.rocks/en/latest/admin.html#customization">the docs for admin customization</a> for more information.
                         ''',
-                        include=settings.DEBUG,
-                    )
+                    ),
+                    include=settings.DEBUG,
                 ),
                 add_models=add_models,
             ),
