@@ -124,6 +124,12 @@ class EditCells(Cells):
 
 
 class EditColumn(Column):
+    # language=rst
+    """
+    The column class for `EditTable`.
+    """
+
+
     edit: Field = Refinable()
 
     def on_refine_done(self):
@@ -181,6 +187,8 @@ def edit_table__post_handler(table, request, **_):
 
     def validate(cells_iterator, form, errors):
         for cells in cells_iterator:
+            if not isinstance(cells, EditCells):
+                continue
             instance = cells.row
             form.instance = instance
             for cell in cells.iter_editable_cells():
@@ -207,6 +215,8 @@ def edit_table__post_handler(table, request, **_):
 
     def save(cells_iterator, form):
         for cells in cells_iterator:
+            if not isinstance(cells, EditCells):
+                continue
             instance = cells.row
             form.instance = instance
             attrs_to_save = []
@@ -236,6 +246,22 @@ def edit_table__post_handler(table, request, **_):
 
 
 class EditTable(Table):
+    # language=rst
+    """
+    Describe an editable table. Example:
+
+    .. code-block:: python
+
+        table = EditTable(
+            auto__model=Album,
+            columns__name__edit__include=True,
+        )
+
+        # @test
+        show_output(table)
+        # @end
+    """
+
     edit_errors = None
     create_errors = None
     edit_form: Form = Refinable()
