@@ -33,6 +33,8 @@ from django.http.response import HttpResponseBase
 from django.template import Context
 from django.utils.functional import Promise
 from django.utils.translation import gettext
+
+from iommi.endpoint import DISPATCH_PREFIX
 from iommi.struct import Struct
 
 from iommi._db_compat import field_defaults_factory
@@ -113,6 +115,7 @@ from iommi.refinable import (
     Refinable,
     refinable,
     RefinableMembers,
+    SpecialEvaluatedRefinable,
 )
 from iommi.shortcut import with_defaults
 from iommi.sort_after import sort_after
@@ -154,10 +157,12 @@ def many_to_many_factory_write_to_instance(field, instance, value):
 _field_factory_by_field_type = {}
 
 
-def register_field_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING):
+def register_field_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING, **kwargs):
     assert shortcut_name is not MISSING or factory is not MISSING
     if factory is MISSING:
-        factory = Shortcut(call_target__attribute=shortcut_name)
+        factory = Shortcut(call_target__attribute=shortcut_name, **kwargs)
+    else:
+        assert not kwargs, 'Can not provide both a factory and additional defaults separately'
 
     _field_factory_by_field_type[django_field_class] = factory
 
@@ -469,6 +474,7 @@ def render_fragment(fragment):
 
 @with_meta
 class Field(Part, Tag):
+    # language=rst
     """
     Class that describes a field, i.e. what input controls to render, the label, etc.
 
@@ -492,12 +498,12 @@ class Field(Part, Tag):
 
     parse_empty_string_as_none: bool = EvaluatedRefinable()
     # parsed_data/parsed_data contains data that has been interpreted, but not checked for validity or access control
-    parsed_data: Any = Refinable()  # parsed_data is evaluated, but in a special way so gets no EvaluatedRefinable type
+    parsed_data: Any = SpecialEvaluatedRefinable()
 
-    initial: Any = Refinable()  # initial is evaluated, but in a special way so gets no EvaluatedRefinable type
+    initial: Any = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
 
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     required: bool = EvaluatedRefinable()
 
     input: Fragment = Refinable()
@@ -507,15 +513,14 @@ class Field(Part, Tag):
 
     is_list: bool = EvaluatedRefinable()
     is_boolean: bool = EvaluatedRefinable()
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
     model_field = Refinable()
     model_field_name = Refinable()
 
     editable: bool = EvaluatedRefinable()
     strip_input: bool = EvaluatedRefinable()
 
-    # choices is evaluated, but in a special way so gets no EvaluatedRefinable type
-    choices: Callable[..., List[Any]] = Refinable()
+    choices: Callable[..., List[Any]] = SpecialEvaluatedRefinable()
     choice_id_formatter: Callable[..., str] = Refinable()
     choice_display_name_formatter: Callable[..., str] = Refinable()
     choice_to_optgroup: Optional[Callable[..., Optional[str]]] = Refinable()
@@ -570,6 +575,7 @@ class Field(Part, Tag):
         empty_label='---',
     )
     def __init__(self, **kwargs):
+        # language=rst
         """
         Note that, in addition to the parameters with the defined behavior below, you can pass in any keyword argument you need yourself, including callables that conform to the protocol, and they will be added and evaluated as members.
 
@@ -580,22 +586,18 @@ class Field(Part, Tag):
                 Field(attrs__id=lambda form, field: 'my_id_%s' % field._name)
 
         :param after: Set the order of columns, see the `howto <https://docs.iommi.rocks/en/latest/cookbook_forms.html#how-do-i-change-the-order-of-the-fields>`_ for an example.
-        :param is_valid: validation function. Should return a tuple of `(bool, reason_for_failure_if_bool_is_false)` or raise ValidationError. Default: `lambda form, field, parsed_data: (True, '')`
-        :param parse: Parse function. Default just returns the string input unchanged: `lambda form, field, string_value: string_value`. This function can raise `ValueError` or `ValidationError` to produce a field error message.
         :param initial: Initial value of the field
         :param attr: The attribute path to apply or get the data from. For example using `foo__bar__baz` will result in `your_instance.foo.bar.baz` will be set by the `apply()` function. Defaults to same as name
         :param attrs: A dict containing any custom html attributes to be sent to the `input__template`.
         :param display_name: The text in the HTML label tag. Default: `capitalize(name).replace('_', ' ')`
-        :param template: django template filename for the entire row. Normally you shouldn't need to override on this level. Prefer overriding `input__template`, `label__template` or `error__template` as needed.
-        :param template_string: You can inline a template string here if it's more convenient than creating a file. Default: `None`
-        :param input__template: Django template filename for the template for just the input control.
-        :param label__template: Django template filename for the template for just the label tab.
+        :param template: Django template filename or `Template` instance for the entire row. Normally you shouldn't need to override on this level. Prefer overriding `input__template`, `label__template` or `error__template` as needed.
+        :param input__template: Django template filename or `Template` instance for the template for just the input control.
+        :param label__template: Django template filename or `Template` instance for the template for just the label tab.
         :param required: If the field is a required field. Default: `True`
         :param help_text: The help text will be grabbed from the django model if specified and available.
 
         :param editable: Is this field editable.
         :param strip_input: Runs the input data through standard python .strip() before passing it to the parse function (can NOT be callable). Default: `True`
-        :param render_value: Render the parsed and validated value into a string. Default just converts to unicode: `lambda form, field, value: unicode(value)`
         :param is_list: Interpret request data as a list (can NOT be a callable). Default: `False``
         :param read_from_instance: Callback to retrieve value from edited instance. Invoked with parameters field and instance.
         :param write_to_instance: Callback to write value to instance. Invoked with parameters field, instance and value.
@@ -614,6 +616,8 @@ class Field(Part, Tag):
         model_field = self.model_field
         if model_field and model_field.remote_field:
             self.model = model_field.remote_field.model
+        elif isinstance(self.model, SpecialEvaluatedRefinable):
+            self.model = None
 
         # value/value_data_list is the final step that contains parsed and valid data
         self.value = None
@@ -641,12 +645,33 @@ class Field(Part, Tag):
     @staticmethod
     @refinable
     def is_valid(form: 'Form', field: 'Field', parsed_data: Any, **_) -> Tuple[bool, str]:
+        # language=rst
+        """
+        Validation function. Should return a tuple of `(bool, reason_for_failure_if_bool_is_false)` or raise ValidationError.
+
+        .. code-block:: python
+
+            form = Form.create(
+                auto__model=Artist,
+                fields__name__is_valid=lambda parsed_data, **_: (parsed_data.startswith('H'), 'Must start with H!'),
+            )
+
+            # @test
+            show_output(form.bind(request=req('post', **{'-submit': '', 'name': 'blizzard of ozz'})))
+            # @end
+        """
+
         return True, ''
 
     # noinspection PyUnusedLocal
     @staticmethod
     @refinable
     def parse(form: 'Form', field: 'Field', string_value: str, **_) -> Any:
+        # language=rst
+        """
+        Parse function. Default just returns the string input unchanged. This function can raise `ValueError` or `ValidationError` to produce a field error message.
+        """
+
         del form, field
         return string_value
 
@@ -658,6 +683,26 @@ class Field(Part, Tag):
     @staticmethod
     @refinable
     def render_value(form: 'Form', field: 'Field', value: Any, **kwargs) -> str:
+        # language=rst
+        """
+        Render the parsed and validated value into a string. Default just converts to `str`.
+
+        .. code-block:: python
+
+            sentinel = '!!custom!!'
+            form = Form(
+                fields__foo=Field(
+                    initial='not sentinel value',
+                    render_value=lambda form, field, value, **_: sentinel,
+                )
+            )
+
+            # @test
+            show_output(form)
+            assert sentinel in form.bind(request=req('get')).__html__()
+            # @end
+        """
+
         if isinstance(value, (list, QuerySet)):
             return ', '.join(field.render_value(form=form, field=field, value=v, **kwargs) for v in value)
         else:
@@ -942,7 +987,7 @@ class Field(Part, Tag):
             return groups
 
     @classmethod
-    def from_model(cls, model, model_field_name=None, model_field=None, **kwargs):
+    def from_model(cls, model=None, model_field_name=None, model_field=None, **kwargs):
         return member_from_model(
             cls=cls,
             model=model,
@@ -1210,6 +1255,14 @@ class Field(Part, Tag):
     def phone_number(cls, **kwargs):
         return cls(**kwargs)
 
+    @classmethod
+    @with_defaults(
+        template=Template(''),
+        editable=False,
+    )
+    def non_rendered(cls, **kwargs):
+        return cls(**kwargs)
+
     @classmethod
     @with_defaults
     def foreign_key(cls, model_field, model, **kwargs):
@@ -1220,6 +1273,21 @@ class Field(Part, Tag):
         )
         return cls.choice_queryset(model_field=model_field, **kwargs)
 
+    @classmethod
+    @with_defaults(
+        editable=False,
+        display_name=lambda field, **_: capitalize(field.model_field.related_model._meta.verbose_name_plural),
+        help_text=None,
+    )
+    def foreign_key_reverse(cls, *, model_field, **kwargs):
+        setdefaults_path(
+            kwargs,
+            choices=lambda field, **_: field.model_field.remote_field.model.objects.all(),
+            read_from_instance=lambda field, instance, **_: getattr_path(instance, field.attr).all(),
+            extra__django_related_field=True,
+        )
+        return cls.multi_choice_queryset(model_field=model_field, **kwargs)
+
     @classmethod
     @with_defaults
     def many_to_many(cls, model_field, **kwargs):
@@ -1232,6 +1300,14 @@ class Field(Part, Tag):
         )
         return cls.multi_choice_queryset(model_field=model_field, **kwargs)
 
+    @classmethod
+    @with_defaults(
+        display_name=lambda field, **_: capitalize(field.model_field.remote_field.model._meta.verbose_name_plural),
+        help_text=None,
+    )
+    def many_to_many_reverse(cls, model_field, **kwargs):
+        return cls.many_to_many(model_field=model_field, **kwargs)
+
 
 def is_django_promise_with_string_proxy(redirect_to):
     return (
@@ -1360,8 +1436,6 @@ class Form(Part):
             ),
         ).bind(request=request)
 
-    See tri.declarative docs for more on this dual style of declaration.
-
     In the common case the fields namespace will contain only instances of `Field`, but
     iommi actually supports arbitrary `Part` objects. For example:
 
@@ -1404,15 +1478,14 @@ class Form(Part):
     actions_template: Union[str, Template] = Refinable()
     # Only for nested forms: The attribute of the parent forms instance to use as this forms instance (default _name)
     attr: str = EvaluatedRefinable()
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     editable: bool = Refinable()
-    # h_tag is evaluated, but in a special way so gets no EvaluatedRefinable type
-    h_tag: Union[Fragment, str] = Refinable()
-    title: Fragment = Refinable()  # title is evaluated, but in a special way so gets no EvaluatedRefinable type
+    h_tag: Union[Fragment, str] = SpecialEvaluatedRefinable()
+    title: Fragment = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     errors: Errors = Refinable()
 
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
     member_class: Type[Field] = Refinable()
     action_class: Type[Action] = Refinable()
     page_class: Type[Page] = Refinable()
@@ -1439,6 +1512,8 @@ class Form(Part):
         attrs__enctype='multipart/form-data',
         h_tag__call_target=Header,
         field_group__call_target=FieldGroup,
+        template='iommi/form/form.html',
+        actions_template='iommi/form/actions.html',
         attr=MISSING,
     )
     def __init__(self, **kwargs):
@@ -1464,22 +1539,23 @@ class Form(Part):
             )
             if auto.instance is not None:
                 self.instance = auto.instance
-            crud_type = self.extra.get('crud_type')
-            if 'title' not in self.iommi_namespace and crud_type is not None:
-                self.title = capitalize(
-                    gettext('%(crud_type)s %(model_name)s')
-                    % dict(crud_type=gettext(crud_type), model_name=model._meta.verbose_name)
-                )
-                # TODO this in extra_member_defaults
-                setdefaults_path(
-                    self.actions,
-                    submit__display_name=gettext('Save') if crud_type == 'edit' else capitalize(gettext(crud_type)),
-                )
 
             self.model = model
         else:
             fields_from_auto = None
 
+        extra_action_defaults = Namespace()
+        crud_type = self.extra.get('crud_type')
+        if 'title' not in self.iommi_namespace and crud_type is not None:
+            self.title = capitalize(
+                gettext('%(crud_type)s %(model_name)s')
+                % dict(crud_type=gettext(crud_type), model_name=(self.model or self.instance)._meta.verbose_name)
+            )
+            extra_action_defaults = setdefaults_path(
+                extra_action_defaults,
+                submit__display_name=gettext('Save') if crud_type == 'edit' else capitalize(gettext(crud_type)),
+            )
+
         # Submit is special.
         # We used to have an automatic action submit button. Now we instead if something is inj
         # the actions submit space assume you want to define it as a primary button (unless you
@@ -1495,6 +1571,7 @@ class Form(Part):
             self,
             name='actions',
             members_from_namespace=self.actions,
+            extra_member_defaults=extra_action_defaults,
             cls=self.get_meta().action_class,
             members_cls=Actions,
         )
@@ -1619,8 +1696,10 @@ class Form(Part):
         return False
 
     def is_valid(self):
-        """Is the form valid?  Can be called inside forms post_validation hook to determine if the
-        individual fields were all valid."""
+        """
+        Is the form valid?  Can be called inside forms post_validation hook to determine if the
+        individual fields were all valid.
+        """
         assert self._is_bound, NOT_BOUND_MESSAGE
         assert self._valid is not None, "Internal error: Once a form is bound we should know if it is valid or not"
         return self._valid
@@ -1655,6 +1734,27 @@ class Form(Part):
         pass
 
     def add_error(self, msg):
+        # language=rst
+        """
+        Explicitly add an error message to the forms global error set.
+
+        Example:
+
+        .. code-block:: python
+
+            def post_validation(form, **_):
+                form.add_error('global error')
+
+            form = Form.create(
+                auto__model=Album,
+                post_validation=post_validation,
+            )
+            # @test
+            form = form.bind(request=req('post', **{'-submit': '',}))
+            assert form.get_errors()['global'] == {'global error'}
+            show_output(form)
+            # @end
+        """
         assert msg
         self._errors.add(msg)
         self._valid = False
@@ -1667,7 +1767,7 @@ class Form(Part):
         r = []
         for group, parts in groupby(values(self.parts), key=lambda x: getattr(x, 'group', MISSING)):
             if group is not MISSING:
-                current_group = self.field_group(group=group).bind(parent=self)
+                current_group = self.field_group(_name=f'iommi_field_group_{group}', group=group).bind(parent=self)
                 r.append(current_group.iommi_open_tag())
 
                 r.extend([part.__html__() for part in parts])
@@ -1679,7 +1779,7 @@ class Form(Part):
         # We need to preserve all other GET parameters, so we can e.g. filter in two forms on the same page, and keep sorting after filtering
         own_field_paths = {f.iommi_path for f in values(self.fields)}
         for k, v in items(self.get_request().GET):
-            if k not in own_field_paths and not k.startswith('-'):
+            if k not in own_field_paths and not k.startswith('-') and not k.startswith(DISPATCH_PREFIX):
                 r.append(format_html('<input type="hidden" name="{}" value="{}" />', k, v))
 
         return format_html('{}\n' * len(r), *r)
@@ -1715,7 +1815,8 @@ class Form(Part):
     @staticmethod
     @refinable
     def write_nested_form_to_instance(form, instance):
-        """Write the nested_form to the instance.
+        """
+        Write the nested_form to the instance.
 
         This is analogous to `Field.write_to_instance` but for nested forms.
         """
@@ -1728,7 +1829,8 @@ class Form(Part):
     @staticmethod
     @refinable
     def read_nested_form_from_instance(form: 'Form', instance: Any) -> Any:
-        """Read the nested forms instance from the parent forms instance.
+        """
+        Read the nested forms instance from the parent forms instance.
 
         This is analogous to `Field.read_from_instance` but for nested forms.
         """
@@ -1746,6 +1848,12 @@ class Form(Part):
             field.write_to_instance(field, instance, field.value)
 
     def get_errors(self):
+        """
+        Get a dict containing two keys:
+
+        - `global` for errors global to the entire form.
+        - `fields` for errors specific to fields. This is itself a dict with a key for each field.
+        """
         assert self._is_bound, NOT_BOUND_MESSAGE
         r = {}
         if self._errors:
@@ -1794,6 +1902,16 @@ class Form(Part):
         actions__submit__post_handler=delete_object__post_handler,
         extra__crud_type='delete',
         editable=False,
+        fields__iommi_default_text=dict(
+            call_target=Fragment,
+            include=lambda form, **_: list(form.iommi_namespace.fields.keys()) == ['iommi_default_text'],
+            after=0,
+            tag='p',
+            children__text=lambda instance, **_: (
+                gettext('Are you sure you want to delete the %(model_name)s "%(instance)s"?')
+                % dict(model_name=instance._meta.verbose_name, instance=str(instance))
+            ),
+        ),
     )
     def delete(cls, **kwargs):
         return cls.crud(**kwargs)
