@@ -25,8 +25,9 @@ from iommi.declarative.namespace import (
 )
 from iommi.declarative.with_meta import with_meta
 from iommi.evaluate import (
+    evaluate_as_needed,
     evaluate_strict,
-    evaluate_strict_container,
+    find_static_items,
 )
 from iommi.member import (
     bind_members,
@@ -44,6 +45,7 @@ from iommi.refinable import (
     Prio,
     Refinable,
     RefinableMembers,
+    SpecialEvaluatedRefinable,
 )
 from iommi.shortcut import with_defaults
 
@@ -125,7 +127,7 @@ class Fragment(Part, Tag):
 
     .. code-block:: python
 
-        # @ test
+        # @test
         from iommi.fragment import Fragment
         # @end
 
@@ -153,7 +155,7 @@ class Fragment(Part, Tag):
     `MyPage(parts__header__tag='h2')` it will be rendered with a `<h2>`.
     """
 
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     tag = EvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     children = RefinableMembers()
@@ -178,8 +180,13 @@ class Fragment(Part, Tag):
             cls=Fragment,
             unknown_types_fall_through=True,
         )
+        if self.children:
+            find_static_items(self.children)
+            self._children_static_items = self.children._static_items
 
-    def render_text_or_children(self, context):
+    def render_text_or_children(self, context=None):
+        if context is None:
+            context = self.get_context()
         request = self.get_request()
         return format_html(
             '{}' * len(self.children),
@@ -198,7 +205,12 @@ class Fragment(Part, Tag):
 
         # Fragment children are special and they can be raw str/int etc but
         # also callables. We need to evaluate them!
-        children = evaluate_strict_container(self.children, **self.iommi_evaluate_parameters())
+
+        _children_static_items = getattr(self, '_children_static_items', None)
+        if _children_static_items:
+            object.__setattr__(self.children, '_static_items', self._children_static_items)
+
+        children = evaluate_as_needed(self.children, self.iommi_evaluate_parameters())
         self.children.update(children)
         self._bound_members.children._bound_members.update(children)
 
