@@ -11,7 +11,7 @@ from iommi._web_compat import (
     get_template_from_string,
     HttpResponse,
     HttpResponseBase,
-    mark_safe,
+    render_template,
     Template,
 )
 from iommi.base import (
@@ -55,6 +55,7 @@ from .refinable import (
     EvaluatedRefinable,
     Refinable,
     RefinableMembers,
+    SpecialEvaluatedRefinable,
 )
 from .sort_after import sort_after
 
@@ -66,7 +67,7 @@ class Part(Traversable):
     See the `howto <https://docs.iommi.rocks/en/latest/cookbook_parts_pages.html#parts-pages>`_ for example usages.
     """
 
-    include: bool = Refinable()  # This is evaluated, but first and in a special way
+    include: bool = SpecialEvaluatedRefinable()
     after: Union[int, str] = EvaluatedRefinable()
     extra: Dict[str, Any] = Refinable()
     # not EvaluatedRefinable because this is an evaluated container so is special
@@ -270,7 +271,7 @@ def as_html(*, request=None, part: PartType, context):
 
         assert not isinstance(context, RequestContext)
         template = part
-        return mark_safe(template.render(context=RequestContext(request, context)))
+        return render_template(template=template, request=request, context=context)
     elif hasattr(part, '__html__'):
         return part.__html__()
     elif part is None:
