@@ -31,7 +31,6 @@ from pyparsing import (
     Word,
     ZeroOrMore,
 )
-from iommi.struct import Struct
 
 from iommi._web_compat import (
     render_template,
@@ -97,11 +96,13 @@ from iommi.refinable import (
     Refinable,
     refinable,
     RefinableMembers,
+    SpecialEvaluatedRefinable,
 )
 from iommi.shortcut import (
     Shortcut,
     with_defaults,
 )
+from iommi.struct import Struct
 
 
 class QueryException(Exception):
@@ -130,10 +131,10 @@ FREETEXT_SEARCH_NAME = 'freetext_search'
 _filter_factory_by_django_field_type = {}
 
 
-def register_filter_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING):
+def register_filter_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING, **kwargs):
     assert shortcut_name is not MISSING or factory is not MISSING
     if factory is MISSING:
-        factory = Shortcut(call_target__attribute=shortcut_name)
+        factory = Shortcut(call_target__attribute=shortcut_name, **kwargs)
 
     _filter_factory_by_django_field_type[django_field_class] = factory
 
@@ -231,7 +232,7 @@ class Filter(Part):
     field: Namespace = Refinable()
     query_operator_for_field: str = EvaluatedRefinable()
     freetext = EvaluatedRefinable()
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
     model_field = Refinable()
     model_field_name = Refinable()
     choices = EvaluatedRefinable()
@@ -284,6 +285,11 @@ class Filter(Part):
     def own_evaluate_parameters(self):
         return dict(filter=self)
 
+    def __html__(self, *, render=None):
+        assert (
+            False
+        ), "Filters aren't rendered directly. You either render the Field corresponding to the filter, or the entire Query object."
+
     @staticmethod
     @refinable
     def query_operator_to_q_operator(op: str) -> str:
@@ -519,6 +525,18 @@ class Filter(Part):
         )
         return cls.choice_queryset(model_field=model_field, **kwargs)
 
+    @classmethod
+    @with_defaults(
+        field__call_target__attribute='foreign_key_reverse',
+    )
+    def foreign_key_reverse(cls, *, model_field, **kwargs):
+        setdefaults_path(
+            kwargs,
+            choices=model_field.remote_field.model.objects.all(),
+            extra__django_related_field=True,
+        )
+        return cls.multi_choice_queryset(model_field=model_field, **kwargs)
+
     @classmethod
     @with_defaults(
         field__call_target__attribute='many_to_many',
@@ -531,6 +549,11 @@ class Filter(Part):
         )
         return cls.multi_choice_queryset(model_field=model_field, **kwargs)
 
+    @classmethod
+    @with_defaults
+    def many_to_many_reverse(cls, model_field, **kwargs):
+        return cls.many_to_many(model_field=model_field, **kwargs)
+
 
 Filter.value_to_q.iommi_needs_attr = True
 
@@ -584,24 +607,25 @@ class Advanced(Fragment):
 @declarative(Filter, '_filters_dict', add_init_kwargs=False)
 @with_meta
 class Query(Part):
+    # language=rst
     """
     Declare a query language. Example:
 
     .. code-block:: python
 
-        class CarQuery(Query):
-            make = Filter.choice(choices=['Toyota', 'Volvo', 'Ford'])
-            model = Filter()
+        class AlbumQuery(Query):
+            year = Filter.integer()
+            name = Filter()
 
-        query_set = Car.objects.filter(
-            CarQuery().bind(request=request).get_q()
+        query_set = Album.objects.filter(
+            AlbumQuery().bind(request=request).get_q()
         )
     """
 
     auto: Namespace = Refinable()
     form: Namespace = Refinable()
     advanced: Namespace = Refinable()
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
     rows = Refinable()
     template: Union[str, Template] = EvaluatedRefinable()
     form_container: Fragment = EvaluatedRefinable()
