@@ -81,7 +81,12 @@ class Style:
 
         for k, v in items(self.config):
             if k[0].islower():
-                warnings.warn(f"Style definition got the keyword {k} which starts with a lowercase letter. Classes should start with an uppercase letter. Either you made a mistake in passing something that won't match any config, or you should rename your class to start with a capital letter.")
+                warnings.warn(
+                    f"Style definition got the keyword {k} which starts with a lowercase letter. "
+                    "Classes should start with an uppercase letter. "
+                    "Either you made a mistake in passing something that won't match any config, "
+                    "or you should rename your class to start with a capital letter."
+                )
 
         sub_style_names = list(sub_styles)
         for base in bases:
@@ -105,6 +110,11 @@ class Style:
         for name, sub_style in items(self.sub_styles):
             sub_style.name = name
 
+        from iommi.debug import iommi_debug_on
+        if iommi_debug_on():
+            import inspect
+            self._instantiated_at_frame = inspect.currentframe().f_back
+
     def resolve(self, obj, is_root=False):
         """
         Calculate the namespaces of additional argument that should be applied
@@ -137,8 +147,9 @@ class Style:
 _styles = {}
 
 
-def register_style(name, style):
-    assert name not in _styles, f'{name} is already registered'
+def register_style(name, style, allow_overwrite=False):
+    if not allow_overwrite:
+        assert name not in _styles, f'{name} is already registered'
     assert style.name is None
     style.name = name
     _styles[name] = style
@@ -189,7 +200,7 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
 
     The `default_classes` parameter can be used to say which classes are
     checked for valid data. By default this is all the `Part`-derived
-    classes in iommmi. This parameter is primarily used by tests.
+    classes in iommi. This parameter is primarily used by tests.
 
     The `styles` parameter can be used to specify which exact styles to
     validate. By default it will validate all registered styles. This
