@@ -70,8 +70,6 @@ base = Style(
             create__actions__submit__display_name=gettext_lazy('Create'),
             delete__actions__submit__display_name=gettext_lazy('Delete'),
         ),
-        template='iommi/form/form.html',
-        actions_template='iommi/form/actions.html',
     ),
     Field=dict(
         shortcuts=dict(
@@ -103,6 +101,10 @@ base = Style(
     Paginator=dict(
         show_always=False,
         template='iommi/table/paginator.html',
+        item__tag='span',
+        container__tag='div',
+        active_item__tag='span',
+        link__tag='a',
     ),
     Query=dict(
         template='iommi/query/form.html',
