@@ -2,7 +2,10 @@ from iommi import html
 from iommi.style import (
     Style,
 )
-from iommi.style_base import base
+from iommi.style_base import (
+    base,
+    select2_enhanced_forms,
+)
 from iommi.style_font_awesome_4 import font_awesome_4
 
 semantic_ui_base = Style(
@@ -133,4 +136,5 @@ semantic_ui_base = Style(
 semantic_ui = Style(
     semantic_ui_base,
     font_awesome_4,
+    select2_enhanced_forms,
 )
