@@ -1,4 +1,5 @@
 import csv
+import warnings
 from datetime import (
     date,
     datetime,
@@ -41,7 +42,10 @@ from django.utils.translation import (
     gettext_lazy,
 )
 from math import ceil
-from iommi.struct import Struct
+from iommi.struct import (
+    merged,
+    Struct,
+)
 
 from iommi._web_compat import (
     format_html,
@@ -64,6 +68,7 @@ from iommi.attrs import (
 )
 from iommi.base import (
     build_as_view_wrapper,
+    capitalize,
     get_display_name,
     items,
     keys,
@@ -132,6 +137,7 @@ from iommi.refinable import (
     refinable,
     RefinableMembers,
     RefinableObject,
+    SpecialEvaluatedRefinable,
 )
 from iommi.shortcut import with_defaults
 from iommi.sort_after import LAST
@@ -145,10 +151,10 @@ LAST = LAST
 _column_factory_by_field_type = {}
 
 
-def register_column_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING):
+def register_column_factory(django_field_class, *, shortcut_name=MISSING, factory=MISSING, **kwargs):
     assert shortcut_name is not MISSING or factory is not MISSING
     if factory is MISSING:
-        factory = Shortcut(call_target__attribute=shortcut_name)
+        factory = Shortcut(call_target__attribute=shortcut_name, **kwargs)
 
     _column_factory_by_field_type[django_field_class] = factory
 
@@ -345,8 +351,9 @@ class Column(Part):
     sortable: bool = EvaluatedRefinable()
     group: Optional[str] = EvaluatedRefinable()
     auto_rowspan: bool = EvaluatedRefinable()
+    row_group: Namespace = EvaluatedRefinable()
     cell: Namespace = Refinable()
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
     model_field = Refinable()
     model_field_name = Refinable()
     choices: Iterable = EvaluatedRefinable()
@@ -383,6 +390,10 @@ class Column(Part):
         header__template='iommi/table/header.html',
         header__url=None,
         render_column=True,
+        row_group__include=False,
+        row_group__template='iommi/table/row_group.html',
+        row_group__tag='th',
+        row_group__attrs__colspan='99',
     )
     def __init__(self, **kwargs):
         """
@@ -420,6 +431,12 @@ class Column(Part):
                 self.iommi_namespace.get('choices') is not None
             ), 'To use Column.choice, you must pass the choices list'
 
+        model_field = self.model_field
+        if model_field and model_field.remote_field:
+            self.model = model_field.remote_field.model
+        elif isinstance(self.model, SpecialEvaluatedRefinable):
+            self.model = None
+
         self.header = HeaderColumnConfig(**self.header).refine_done(parent=self)
         self.is_sorting: bool = None
         self.sort_direction: str = None
@@ -477,7 +494,7 @@ class Column(Part):
         filter__call_target__attribute='from_model',
         bulk__call_target__attribute='from_model',
     )
-    def from_model(cls, model, model_field_name=None, model_field=None, **kwargs):
+    def from_model(cls, model=None, model_field_name=None, model_field=None, **kwargs):
         return member_from_model(
             cls=cls,
             model=model,
@@ -501,10 +518,22 @@ class Column(Part):
         extra__icon_attrs__style=EMPTY,
     )
     def icon(cls, *args, **kwargs):
+        # language=rst
         """
         Shortcut to create font awesome-style icons.
 
         :param extra__icon: the font awesome name of the icon
+
+        .. code-block:: python
+
+            table = Table(
+                auto__model=Album,
+                columns__icon=Column(extra__icon='music'),
+            )
+
+            # @test
+            show_output(table)
+            # @end
         """
         assert len(args) in (0, 1), "You can only pass 1 positional argument: icon, or you can pass no arguments."
 
@@ -522,8 +551,20 @@ class Column(Part):
         display_name=gettext_lazy('Edit'),
     )
     def edit(cls, **kwargs):
+        # language=rst
         """
         Shortcut for creating a clickable edit icon. The URL defaults to `your_object.get_absolute_url() + 'edit/'`. Specify the option cell__url to override.
+
+        .. code-block:: python
+
+            table = Table(
+                auto__model=Album,
+                columns__edit=Column.edit(after=0),
+            )
+
+            # @test
+            show_output(table)
+            # @end
         """
         return cls.icon(**kwargs)
 
@@ -533,9 +574,22 @@ class Column(Part):
         display_name=gettext_lazy('Delete'),
     )
     def delete(cls, **kwargs):
+        # language=rst
         """
         Shortcut for creating a clickable delete icon. The URL defaults to `your_object.get_absolute_url() + 'delete/'`. Specify the option cell__url to override.
+
+        .. code-block:: python
+
+            table = Table(
+                auto__model=Album,
+                columns__delete=Column.delete(),
+            )
+
+            # @test
+            show_output(table)
+            # @end
         """
+
         return cls.icon(**kwargs)
 
     @classmethod
@@ -545,8 +599,21 @@ class Column(Part):
         display_name=gettext_lazy('Download'),
     )
     def download(cls, **kwargs):
+        # language=rst
         """
         Shortcut for creating a clickable download icon. The URL defaults to `your_object.get_absolute_url() + 'download/'`. Specify the option cell__url to override.
+
+
+        .. code-block:: python
+
+            table = Table(
+                auto__model=Album,
+                columns__download=Column.download(),
+            )
+
+            # @test
+            show_output(table)
+            # @end
         """
         return cls.icon(**kwargs)
 
@@ -556,8 +623,21 @@ class Column(Part):
         display_name=gettext_lazy('Run'),
     )
     def run(cls, **kwargs):
+        # language=rst
         """
         Shortcut for creating a clickable run icon. The URL defaults to `your_object.get_absolute_url() + 'run/'`. Specify the option cell__url to override.
+
+
+        .. code-block:: python
+
+            table = Table(
+                auto__model=Album,
+                columns__run=Column.run(),
+            )
+
+            # @test
+            show_output(table)
+            # @end
         """
         return cls.icon(**kwargs)
 
@@ -568,11 +648,36 @@ class Column(Part):
         filter__is_valid_filter=lambda **_: (True, ''),
         filter__field__include=False,
         attr=None,
+        cell__value=lambda table, cells, row, **_: (
+            row.pk
+            if isinstance(table.rows, QuerySet)
+            else
+            # row_index is the visible row number
+            # See selection() for the code that does the lookup
+            cells.row_index
+        ),
+        cell__format=lambda column, row, value, **kwargs: format_html(
+            # language=HTML
+            '<input type="checkbox" class="checkbox" name="{checkbox_name}_{row_id}" {checked_str} />',
+            checkbox_name=column.extra.checkbox_name,
+            row_id=value,
+            checked_str=(
+                'checked'
+                if evaluate_strict(column.extra.checked, column=column, row=row, value=value, **kwargs)
+                else ''
+            ),
+        ),
+        extra__checkbox_name='pk',
+        extra__checked=lambda **_: False,
     )
-    def select(cls, checkbox_name='pk', checked=lambda row, **_: False, **kwargs):
+    def select(cls, *, checkbox_name=None, checked=None, **kwargs):
+        # language=rst
         """
         Shortcut for a column of checkboxes to select rows. This is useful for implementing bulk operations.
 
+        By default tables have a column named `select` that is hidden that is used for this purpose, so you only
+        need to turn it on to get it. See the example below.
+
         To implement a custom post handler that operates on the selected rows, do
 
          .. code-block:: python
@@ -583,25 +688,31 @@ class Column(Part):
                 # matching the type of rows of the table
                 ...
 
-            Table(.... ,
-                bulk__actions=Action.submit(post_handler=my_handler)
+            table = Table(
+                auto__model=Album,
+                columns__select__include=True,
+                bulk__actions__submit=Action.submit(post_handler=my_handler)
             )
 
-        :param checkbox_name: the name of the checkbox. Default is `"pk"`, resulting in checkboxes like `"pk_1234"`.
-        :param checked: callable to specify if the checkbox should be checked initially. Defaults to `False`.
+            # @test
+            show_output(table)
+            # @end
+
+        :param extra__checkbox_name: the name of the checkbox. Default is `"pk"`, resulting in checkboxes like `"pk_1234"`.
+        :param extra__checked: callable to specify if the checkbox should be checked initially. Defaults to `False`.
         """
-
-        def cell__value(row, table, cells, **kwargs):
-            checked_str = ' checked' if evaluate_strict(checked, row=row, **kwargs) else ''
-            if isinstance(table.rows, QuerySet):
-                row_id = row.pk
-            else:
-                # row_index is the visible row number
-                # See selection() for the code that does the lookup
-                row_id = cells.row_index
-            return mark_safe(f'<input type="checkbox"{checked_str} class="checkbox" name="{checkbox_name}_{row_id}" />')
-
-        setdefaults_path(kwargs, dict(cell__value=cell__value))
+        if checkbox_name is not None:
+            warnings.warn(
+                'The preferred way to specify checkbox name is renamed to extra__checkbox_name',
+                category=DeprecationWarning,
+            )
+            setdefaults_path(kwargs, extra__checkbox_name=checkbox_name)
+        if checked is not None:
+            warnings.warn(
+                'The preferred way to specify checked callback is renamed to extra__checked',
+                category=DeprecationWarning,
+            )
+            setdefaults_path(kwargs, extra__checked=checked)
         return cls(**kwargs)
 
     @classmethod
@@ -612,8 +723,26 @@ class Column(Part):
         cell__format=lambda value, **_: mark_safe('<i class="fa fa-check" title="Yes"></i>') if value else '',
     )
     def boolean(cls, **kwargs):
+        # language=rst
         """
         Shortcut to render booleans as a check mark if true or blank if false.
+
+
+        .. code-block:: python
+
+            table = Table(
+                columns__name=Column(),
+                columns__boolean=Column.boolean(),
+                rows=[
+                    Struct(name='true!', boolean=True),
+                    Struct(name='false!', boolean=False),
+                ]
+            )
+
+            # @test
+            show_output(table)
+            # @end
+
         """
         return cls(**kwargs)
 
@@ -622,6 +751,11 @@ class Column(Part):
         filter__call_target__attribute='boolean_tristate',
     )
     def boolean_tristate(cls, **kwargs):
+        # language=rst
+        """
+        This shortcut sets up `boolean_tristate` for the filter.
+        """
+
         return cls.boolean(**kwargs)
 
     @classmethod
@@ -630,6 +764,11 @@ class Column(Part):
         filter__call_target__attribute='choice',
     )
     def choice(cls, **kwargs):
+        # language=rst
+        """
+        This shortcut sets up `choices` for the filter and bulk form.
+        """
+
         instance = cls(**kwargs)
         instance = instance.refine(
             Prio.shortcut,
@@ -644,6 +783,10 @@ class Column(Part):
         filter__call_target__attribute='choice_queryset',
     )
     def choice_queryset(cls, **kwargs):
+        # language=rst
+        """
+        This shortcut sets up `choices` for the filter and bulk form for the choice queryset case.
+        """
         setdefaults_path(
             kwargs,
             dict(
@@ -659,6 +802,10 @@ class Column(Part):
         filter__call_target__attribute='multi_choice_queryset',
     )
     def multi_choice_queryset(cls, **kwargs):
+        # language=rst
+        """
+        This shortcut sets up `choices` for the filter and bulk form for the multi choice queryset case.
+        """
         setdefaults_path(
             kwargs,
             dict(
@@ -674,6 +821,10 @@ class Column(Part):
         filter__call_target__attribute='multi_choice',
     )
     def multi_choice(cls, **kwargs):
+        # language=rst
+        """
+        This shortcut sets up `choices` for the filter and bulk form for the multi choice case.
+        """
         setdefaults_path(
             kwargs,
             dict(
@@ -689,6 +840,10 @@ class Column(Part):
         filter__call_target__attribute='text',
     )
     def text(cls, **kwargs):
+        # language=rst
+        """
+        This is an explicit synonym for `Column()`.
+        """
         return cls(**kwargs)
 
     @classmethod
@@ -699,7 +854,10 @@ class Column(Part):
     @classmethod
     @with_defaults
     def link(cls, **kwargs):
-        # Shortcut for creating a cell that is a link. The URL is the result of calling `get_absolute_url()` on the object.
+        # language=rst
+        """
+        Shortcut for creating a cell that is a link. The URL is the result of calling `get_absolute_url()` on the object.
+        """
         def link_cell_url(column, row, **_):
             r = getattr_path(row, column.attr)
             return r.get_absolute_url() if r else ''
@@ -808,10 +966,19 @@ class Column(Part):
         setdefaults_path(
             kwargs,
             choices=model_field.remote_field.model.objects.all(),
-            model_field=model_field.remote_field,
+            model_field=model_field,
         )
         return cls.multi_choice_queryset(**kwargs)
 
+    @classmethod
+    @with_defaults(
+        bulk__call_target__attribute='many_to_many_reverse',
+        filter__call_target__attribute='many_to_many_reverse',
+        display_name=lambda column, **_: capitalize(column.model_field.remote_field.model._meta.verbose_name_plural),
+    )
+    def many_to_many_reverse(cls, model_field, **kwargs):
+        return cls.many_to_many(model_field=model_field, **kwargs)
+
     @classmethod
     @with_defaults(
         bulk__call_target__attribute='foreign_key',
@@ -832,6 +999,23 @@ class Column(Part):
         )
         return cls.choice_queryset(model_field=model_field, **kwargs)
 
+    @classmethod
+    @with_defaults(
+        bulk__call_target__attribute='foreign_key_reverse',
+        filter__call_target__attribute='foreign_key_reverse',
+        cell__format=lambda value, **_: ', '.join(['%s' % x for x in value.all()]),
+        data_retrieval_method=DataRetrievalMethods.prefetch,
+        sortable=False,
+        extra__django_related_field=True,
+        display_name=lambda column, **_: capitalize(column.model_field.remote_field.model._meta.verbose_name_plural),
+    )
+    def foreign_key_reverse(cls, *, model_field, **kwargs):
+        setdefaults_path(
+            kwargs,
+            choices=model_field.remote_field.model.objects.all(),
+        )
+        return cls.multi_choice_queryset(model_field=model_field, **kwargs)
+
 
 @with_meta
 class Cells(Traversable, Tag):
@@ -840,7 +1024,7 @@ class Cells(Traversable, Tag):
     """
 
     template: Union[str, Template] = EvaluatedRefinable()
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     tag: str = EvaluatedRefinable()
     extra: Dict[str, Any] = Refinable()
     # not EvaluatedRefinable because this is an evaluated container so is special
@@ -890,6 +1074,18 @@ class Cells(Traversable, Tag):
         return self.cell_class(cells=self, column=column).refine_done(parent=self)
 
 
+class RowGroup(Fragment):
+    def __init__(self, *, value, **kwargs):
+        super(RowGroup, self).__init__(**kwargs)
+        self.value = value
+
+    def get_context(self):
+        return {**super(RowGroup, self).get_context(), 'value': self.value}
+
+    def own_evaluate_parameters(self):
+        return dict(row_group=self)
+
+
 class CellConfig(RefinableObject, Tag):
     url: str = Refinable()
     url_title: str = Refinable()
@@ -923,7 +1119,13 @@ class Cell(CellConfig):
         self.row = cells.row
 
     def on_refine_done(self):
-        self._evaluate_parameters = {**self.cells.iommi_evaluate_parameters(), 'column': self.column}
+        self._evaluate_parameters = merged(
+            self.column.iommi_evaluate_parameters(),
+            cells=self.cells,
+            column=self.column,
+            row=self.row,
+            bound_cell=self,
+        )
 
         self.value = evaluate_strict(self.value, **self._evaluate_parameters)
         self._evaluate_parameters['value'] = self.value
@@ -942,9 +1144,7 @@ class Cell(CellConfig):
     def __html__(self):
         cell__template = self.column.cell.template
         if cell__template:
-            context = dict(
-                table=self.table, column=self.column, cells=self.cells, row=self.row, value=self.value, bound_cell=self
-            )
+            context = self._evaluate_parameters
             return render_template(self.table.get_request(), cell__template, context)
 
         if self.tag:
@@ -967,9 +1167,7 @@ class Cell(CellConfig):
         return cell_contents
 
     def render_formatted(self):
-        return evaluate_strict(
-            self.column.cell.format, row=self.row, value=self.value, **self.column.iommi_evaluate_parameters()
-        )
+        return evaluate_strict(self.column.cell.format, **self._evaluate_parameters)
 
     def __str__(self):
         return self.__html__()
@@ -989,7 +1187,7 @@ class TemplateConfig(RefinableObject):
 
 
 class HeaderConfig(Traversable):
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     extra: Dict[str, Any] = Refinable()
     extra_evaluated: Dict[str, Any] = Refinable()
@@ -1003,13 +1201,13 @@ class HeaderConfig(Traversable):
 
 
 class HeaderColumnConfig(Traversable):
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     url = EvaluatedRefinable()
 
 
 class RowConfig(RefinableObject, Tag):
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     tag = Refinable()
     template: Union[str, Template] = Refinable()
     extra: Dict[str, Any] = Refinable()
@@ -1131,7 +1329,7 @@ def bulk_delete__post_handler(table, form, **_):
             auto__rows=queryset,
             columns__select=dict(
                 include=True,
-                checked=True,
+                extra__checked=True,
                 bulk__include=True,
                 bulk__attrs__style__display='none',
             ),
@@ -1173,18 +1371,17 @@ def paginator__count(rows, **_):
 
 @with_meta
 class Paginator(Traversable):
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     container = Refinable()
-    page: int = Refinable()  # page is evaluated, but in a special way so gets no EvaluatedRefinable type
+    page: int = SpecialEvaluatedRefinable()
     active_item = Refinable()
     item = Refinable()
     link = Refinable()
     adjacent_pages: int = Refinable()
     min_page_size: int = Refinable()
-    # number_of_pages is evaluated, but in a special way so gets no EvaluatedRefinable type
-    number_of_pages: int = Refinable()
-    count: int = Refinable()  # count is evaluated, but in a special way so gets no EvaluatedRefinable type
+    number_of_pages: int = SpecialEvaluatedRefinable()
+    count: int = SpecialEvaluatedRefinable()
     slice = Refinable()
     show_always = Refinable()
 
@@ -1231,6 +1428,11 @@ class Paginator(Traversable):
         self.item.attrs = evaluate_attrs(self.item)
         self.link.attrs = evaluate_attrs(self.link)
 
+        self.container.tag = evaluate_strict(self.container.tag)
+        self.active_item.tag = evaluate_strict(self.active_item.tag)
+        self.item.tag = evaluate_strict(self.item.tag)
+        self.link.tag = evaluate_strict(self.link.tag)
+
         rows = table.sorted_and_filtered_rows
         evaluate_parameters = dict(
             page_size=self.page_size,
@@ -1240,6 +1442,7 @@ class Paginator(Traversable):
 
         if self.page_size is None:
             self.number_of_pages = 1
+            self.count = None
         else:
             self.count = evaluate_strict(self.count, **evaluate_parameters) if rows is not None else 0
             if self.count is None:
@@ -1375,7 +1578,8 @@ def endpoint__csv(table, **_):
 
     def rows():
         for cells in table.cells_for_rows(paginate=False):
-            yield [cell_value(cells, bound_column) for bound_column in columns]
+            if isinstance(cells, Cells):
+                yield [cell_value(cells, bound_column) for bound_column in columns]
 
     def write_csv_row(writer, row):
         row_strings = [smart_text2(value) for value in row]
@@ -1409,19 +1613,26 @@ class _Lazy_tbody:
 @declarative(Column, '_columns_dict', add_init_kwargs=False)
 @with_meta
 class Table(Part, Tag):
+    # language=rst
     """
     Describe a table. Example:
 
     .. code-block:: python
 
-        class FooTable(Table):
-            a = Column()
-            b = Column()
+        class AlbumTable(Table):
+            name = Column()
+            artist = Column()
 
             class Meta:
                 sortable = False
-                attrs__style = 'background: green'
 
+        # @test
+        artist = Artist.objects.create(name='Black Sabbath')
+        Album.objects.create(name='Heaven & Hell', artist=artist, year=1980),
+        Album.objects.create(name='Mob Rules', artist=artist, year=1981),
+
+        show_output(AlbumTable(rows=Album.objects.all()))
+        # @end
     """
 
     query = Refinable()
@@ -1430,18 +1641,16 @@ class Table(Part, Tag):
     sortable: bool = EvaluatedRefinable()
     query_from_indexes: bool = Refinable()
     default_sort_order = Refinable()
-    # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
-    attrs: Attrs = Refinable()
+    attrs: Attrs = SpecialEvaluatedRefinable()
     template: Union[str, Template] = EvaluatedRefinable()
     tag: str = EvaluatedRefinable()
-    # h_tag is evaluated, but in a special way so gets no EvaluatedRefinable type
-    h_tag: Union[Fragment, str] = Refinable()
-    title: str = Refinable()  # title is evaluated, but in a special way so gets no EvaluatedRefinable type
+    h_tag: Union[Fragment, str] = SpecialEvaluatedRefinable()
+    title: str = SpecialEvaluatedRefinable()
     row: RowConfig = EvaluatedRefinable()
     cell: CellConfig = EvaluatedRefinable()
     header = Refinable()
-    model: Type[Model] = Refinable()  # model is evaluated, but in a special way so gets no EvaluatedRefinable type
-    rows = Refinable()  # rows is evaluated, but in a special way so gets no EvaluatedRefinable type
+    model: Type[Model] = SpecialEvaluatedRefinable()
+    rows = SpecialEvaluatedRefinable()
     actions: Dict[str, Action] = RefinableMembers()
     parts: Namespace = RefinableMembers()
     bulk: Optional[Form] = EvaluatedRefinable()
@@ -1461,6 +1670,7 @@ class Table(Part, Tag):
     action_class: Type[Action] = Refinable()
     page_class: Type[Page] = Refinable()
     cells_class: Type[Cells] = Refinable()
+    row_group_class: Type[RowGroup] = Refinable()
 
     empty_message: str = Refinable()
     invalid_form_message: str = Refinable()
@@ -1478,6 +1688,7 @@ class Table(Part, Tag):
         action_class = Action
         page_class = Page
         cells_class = Cells
+        row_group_class = RowGroup
         endpoints__tbody__func = lambda table, **_: {
             'html': table.__html__(template='iommi/table/table_container.html')
         }
@@ -1795,6 +2006,21 @@ class Table(Part, Tag):
             self.query = self.query.refine_done(parent=self)
 
         self.bulk_container = self.bulk_container(_name='bulk_container').refine_done(parent=self)
+        self.container = self.container(_name='container').refine_done(parent=self)
+
+        self.outer = (
+            self.outer(_name='outer').refine(
+                children=dict(
+                    h_tag__template=Template('{{ table.h_tag|default_if_none:"" }}'),
+                    query__template=Template('{{ table.query|default_if_none:"" }}'),
+                    actions=dict(
+                        after=LAST if self.actions_below else 'h_tag',
+                        template=Template('{{ table.render_actions|default_if_none:"" }}'),
+                    ),
+                    container__template=Template('{{ table.container|default_if_none:"" }}'),
+                ),
+            )
+        ).refine_done(parent=self)
 
         super(Table, self).on_refine_done()
 
@@ -1823,23 +2049,9 @@ class Table(Part, Tag):
         build_and_bind_h_tag(self)
 
         self.tbody = self.tbody(_name='tbody', children__text=_Lazy_tbody(self)).bind(parent=self)
-        self.container = self.container(_name='container').bind(parent=self)
+        self.container = self.container.bind(parent=self)
 
-        self.outer = (
-            self.outer(_name='outer')
-            .refine(
-                children=dict(
-                    h_tag__template=Template('{{ table.h_tag|default_if_none:"" }}'),
-                    query__template=Template('{{ table.query|default_if_none:"" }}'),
-                    actions=dict(
-                        after=LAST if self.actions_below else 'h_tag',
-                        template=Template('{{ table.render_actions|default_if_none:"" }}'),
-                    ),
-                    container__template=Template('{{ table.container|default_if_none:"" }}'),
-                ),
-            )
-            .bind(parent=self)
-        )
+        self.outer = self.outer.bind(parent=self)
 
         self.header = self.header.bind(parent=self)
 
@@ -2080,9 +2292,23 @@ class Table(Part, Tag):
         else:
             rows = self.sorted_and_filtered_rows
         preprocessed_rows = self.invoke_callback(self.preprocess_rows, rows=rows)
+
+        row_groups = [c for c in values(self.columns) if c.row_group.include]
+        row_group_values = {c._name: None for c in row_groups}
+
         for i, row in enumerate(preprocessed_rows):
             row = self.invoke_callback(self.preprocess_row, row=row)
             assert row is not None, 'preprocess_row must return the row'
+
+            for column in row_groups:
+                v = getattr_path(row, column.attr)
+                old_value = row_group_values[column._name]
+                if old_value != v:
+                    # noinspection PyCallingNonCallable
+                    yield self.row_group_class(**column.row_group, value=v).bind(parent=self).__html__()
+                row_group_values[column._name] = v
+
+            # noinspection PyCallingNonCallable
             yield self.cells_class(row=row, row_index=i, **self.row.as_dict()).bind(parent=self)
 
     @classmethod
