@@ -5,27 +5,32 @@ from typing import (
     List,
 )
 
-from iommi.attrs import evaluate_attrs
+from iommi.attrs import (
+    evaluate_attrs,
+)
 from iommi.base import (
     items,
     NOT_BOUND_MESSAGE,
 )
 from iommi.declarative.namespace import Namespace
 from iommi.evaluate import (
+    evaluate_as_needed,
     evaluate_members,
     evaluate_strict,
-    evaluate_strict_container,
+    find_static_items,
     get_callable_description,
     get_signature,
+    is_callable,
     matches,
     signature_from_kwargs,
 )
 from iommi.refinable import (
     evaluated_refinable,
     EvaluatedRefinable,
-    is_evaluated_refinable,
+    get_special_evaluated_attributes,
     Prio,
     Refinable,
+    refinable,
     RefinableMembers,
     RefinableObject,
 )
@@ -58,6 +63,11 @@ class Traversable(RefinableObject):
     assets = RefinableMembers()
     endpoints = RefinableMembers()
 
+    @staticmethod
+    @refinable
+    def extra_params(**_):
+        return {}
+
     _bound_members: Dict[str, 'Traversable']
 
     def __init__(self, _name=None, **kwargs):
@@ -138,6 +148,19 @@ class Traversable(RefinableObject):
         result.iommi_style = iommi_style
         return result
 
+    def on_refine_done(self, parent=None):
+        super(Traversable, self).on_refine_done()
+
+        attrs = getattr(self, 'attrs', None)
+        if attrs:
+            find_static_items(attrs)
+            find_static_items(attrs.get('style', None))
+            find_static_items(attrs.get('class', None))
+
+        extra_evaluated = getattr(self, 'extra_evaluated', None)
+        if extra_evaluated:
+            find_static_items(extra_evaluated)
+
     def bind(self, *, parent=None, request=None):
         assert parent is None or parent._is_bound
         assert not self._is_bound
@@ -171,7 +194,11 @@ class Traversable(RefinableObject):
         if parent is None:
             evaluate_parameters['request'] = request
             if hasattr(request, 'iommi_view_params'):
-                evaluate_parameters['params'] = request.iommi_view_params
+                params = request.iommi_view_params
+                extra_params = result.extra_params(request=request, **params)
+                assert isinstance(extra_params, dict), 'extra_params needs to return a dict with additional parameters'
+                params.update(extra_params)
+                evaluate_parameters['params'] = params
         result._evaluate_parameters = evaluate_parameters
 
         if hasattr(result, 'include'):
@@ -187,14 +214,18 @@ class Traversable(RefinableObject):
         if result.include is False:
             return None
 
+        evaluate_members(result, **evaluate_parameters)
+
         if hasattr(result, 'attrs'):
             result.attrs = evaluate_attrs(result, **result.iommi_evaluate_parameters())
 
-        evaluated_attributes = [k for k, v in items(result.get_declared('refinable')) if is_evaluated_refinable(v)]
-        evaluate_members(result, evaluated_attributes, **evaluate_parameters)
-
         if hasattr(result, 'extra_evaluated'):
-            result.extra_evaluated = evaluate_strict_container(result.extra_evaluated or {}, **evaluate_parameters)
+            result.extra_evaluated = Struct(evaluate_as_needed(result.extra_evaluated or {}, evaluate_parameters))
+
+        for k in get_special_evaluated_attributes(result):
+            v = getattr(result, k)
+            if is_callable(v) and not isinstance(v, type):
+                assert False, ('SpecialEvaluatedRefinable not evaluated', k, v, repr(result))
 
         return result
 
